"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
const cluster_1 = __importDefault(require("cluster"));
const config_js_1 = require("./config.js");
const logger_js_1 = require("./logger.js");
// NOTE: Impure function, mutate config by adding gridUrl prop
async function startWebdriverServer(config, options) {
    if (config.useDocker) {
        return (await import('./docker.js')).initDocker(config, options.browser, async () => (await import('./selenium/selenoid.js')).startSelenoidContainer(config, options.debug));
    }
    else {
        return (await import('./selenium/selenoid.js')).startSelenoidStandalone(config, options.debug);
    }
}
async function default_1(options) {
    const config = await (0, config_js_1.readConfig)(options);
    const { browser = config_js_1.defaultBrowser, tests, update, ui, port } = options;
    // NOTE: We don't need docker nor selenoid for webpack or update options
    if (!(config.gridUrl || Object.values(config.browsers).every(({ gridUrl }) => gridUrl)) &&
        !tests &&
        !update) {
        await startWebdriverServer(config, options);
    }
    switch (true) {
        case Boolean(update): {
            (await import('./update.js')).update(config, typeof update == 'string' ? update : undefined);
            return;
        }
        case cluster_1.default.isPrimary: {
            logger_js_1.logger.info('Starting Master Process');
            const resolveApi = (await import('./master/server.js')).start(config.reportDir, port, ui);
            return (await import('./master/index.js')).start(config, options, resolveApi);
        }
        default: {
            logger_js_1.logger.info(`Starting Worker for ${browser}`);
            return (await import('./worker/index.js')).start(config, {
                ...options,
                browser,
            });
        }
    }
}
//# sourceMappingURL=index.js.map